/* sdloshw.c: Top-level SDL management functions.
 *
 * Copyright (C) 2001-2006 by Brian Raiter, under the GNU General Public
 * License. No warranty. See COPYING for details.
 */

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<SDL.h>
#include	"sdlgen.h"
#include	"../err.h"



/* Values global to this library.
 */
oshwglobals	sdlg;

/* This is an automatically-generated file, which contains a
 * representation of the program's icon.
 */
#include	"ccicon.c"

//DKS - modified
/* Dispatch all events sitting in the SDL event queue.
 */
//static void _eventupdate(int wait)
//{
//    static int	mouselastx = -1, mouselasty = -1;
//    SDL_Event	event;
//
//    if (wait)
//	SDL_WaitEvent(NULL);
//    SDL_PumpEvents();
//    while (SDL_PeepEvents(&event, 1, SDL_GETEVENT, SDL_ALLEVENTS)) {
//	switch (event.type) {
//	  case SDL_KEYDOWN:
//	    if (windowmappos(mouselastx, mouselasty) < 0)
//		SDL_ShowCursor(SDL_DISABLE);
//	    keyeventcallback(event.key.keysym.sym, TRUE);
//	    if (event.key.keysym.unicode
//			&& event.key.keysym.unicode != event.key.keysym.sym) {
//		keyeventcallback(event.key.keysym.unicode, TRUE);
//		keyeventcallback(event.key.keysym.unicode, FALSE);
//	    }
//	    break;
//	  case SDL_KEYUP:
//	    if (windowmappos(mouselastx, mouselasty) < 0)
//		SDL_ShowCursor(SDL_DISABLE);
//	    keyeventcallback(event.key.keysym.sym, FALSE);
//	    break;
//	  case SDL_MOUSEBUTTONDOWN:
//	  case SDL_MOUSEBUTTONUP:
//	    SDL_ShowCursor(SDL_ENABLE);
//	    mouselastx = event.motion.x;
//	    mouselasty = event.motion.y;
//	    mouseeventcallback(event.button.x, event.button.y,
//			       event.button.button,
//			       event.type == SDL_MOUSEBUTTONDOWN);
//	    break;
//	  case SDL_MOUSEMOTION:
//	    SDL_ShowCursor(SDL_ENABLE);
//	    mouselastx = event.motion.x;
//	    mouselasty = event.motion.y;
//	    break;
//	  case SDL_QUIT:
//	    exit(EXIT_SUCCESS);
//	}
//    }
//}
static void _eventupdate(int wait)
{


    static int	mouselastx = -1, mouselasty = -1;
    SDL_Event	event;

    if (wait) {

//DKS This won't do if we have to update sounds regularly
//	SDL_WaitEvent(NULL);
		int eventoccured = 0;
		while (!eventoccured) {
			updatesound();
			eventoccured = SDL_PollEvent(NULL);
			if (!eventoccured) {
				SDL_Delay(10);
			}
		}
    } else {
    	updatesound();
    }
    
	
    SDL_PumpEvents();
    while (SDL_PeepEvents(&event, 1, SDL_GETEVENT, SDL_ALLEVENTS)) {
	switch (event.type) {
	  case SDL_KEYDOWN:
	    if (windowmappos(mouselastx, mouselasty) < 0)
		SDL_ShowCursor(SDL_DISABLE);
	    keyeventcallback(event.key.keysym.sym, TRUE);
	    if (event.key.keysym.unicode
			&& event.key.keysym.unicode != event.key.keysym.sym) {
		keyeventcallback(event.key.keysym.unicode, TRUE);
		keyeventcallback(event.key.keysym.unicode, FALSE);
	    }
	    break;
	  case SDL_KEYUP:
	    if (windowmappos(mouselastx, mouselasty) < 0)
		SDL_ShowCursor(SDL_DISABLE);
	    keyeventcallback(event.key.keysym.sym, FALSE);
	    break;

        //new joystick stuff
      case SDL_JOYBUTTONDOWN:
	    if (windowmappos(mouselastx, mouselasty) < 0)
		SDL_ShowCursor(SDL_DISABLE);
	    joybuttoneventcallback(event.jbutton.button, TRUE);
	    break;
	  case SDL_JOYBUTTONUP:
	    if (windowmappos(mouselastx, mouselasty) < 0)
		SDL_ShowCursor(SDL_DISABLE);
	    joybuttoneventcallback(event.jbutton.button, FALSE);
	    break;

#ifndef PLATFORM_GP2X
	  case SDL_MOUSEBUTTONDOWN:
	  case SDL_MOUSEBUTTONUP:
	    SDL_ShowCursor(SDL_ENABLE);
	    mouselastx = event.motion.x;
	    mouselasty = event.motion.y;
	    mouseeventcallback(event.button.x, event.button.y,
			       event.button.button,
			       event.type == SDL_MOUSEBUTTONDOWN);
	    break;
	  case SDL_MOUSEMOTION:
	    SDL_ShowCursor(SDL_ENABLE);
	    mouselastx = event.motion.x;
	    mouselasty = event.motion.y;
	    break;
#endif // PLATFORM_GP2X
	  case SDL_QUIT:
	    exit(EXIT_SUCCESS);
	}
    }
}


//DKS - modified
/* Alter the window decoration.
 */
//void setsubtitle(char const *subtitle)
//{
//    char	buf[270];
//
//    if (subtitle && *subtitle) {
//	sprintf(buf, "Tile World - %.255s", subtitle);
//	SDL_WM_SetCaption(buf, "Tile World");
//    } else {
//	SDL_WM_SetCaption("Tile World", "Tile World");
//    }
//}
void setsubtitle(char const *subtitle)
{
#ifdef PLATFORM_PC
	//dks debug
	printf("shouldn't happen on GP2X\n");

    char	buf[270];

    if (subtitle && *subtitle) {
	sprintf(buf, "Tile World - %.255s", subtitle);
	SDL_WM_SetCaption(buf, "Tile World");
    } else {
	SDL_WM_SetCaption("Tile World", "Tile World");
    }
#endif //PLATFORM_PC
}

//DKS - modified, we're going to explicity shutdown SDL, not with atexit()
/* Shut down SDL.
 */
//static void shutdown(void)
//{
//    SDL_Quit();
//}
static void shutdown(void)
{
//    // Close josytick if opened
//    if(SDL_JoystickOpened(0))
//        SDL_JoystickClose(sdlg.joy);
//
//    SDL_Quit();
}




//DKS - modified
/* Initialize SDL, create the program's icon, and then initialize
 * the other modules of the library.
 */
//int oshwinitialize(int silence, int soundbufsize,
//		   int showhistogram, int fullscreen)
//{
//    SDL_Surface	       *icon;
//
//    sdlg.eventupdatefunc = _eventupdate;
//
//    if (SDL_Init(SDL_INIT_VIDEO) < 0) {
//	errmsg(NULL, "Cannot initialize SDL system: %s\n", SDL_GetError());
//	return FALSE;
//    }
//    atexit(shutdown);
//
//    setsubtitle(NULL);
//
//    icon = SDL_CreateRGBSurfaceFrom(cciconimage, CXCCICON, CYCCICON,
//				    32, 4 * CXCCICON,
//				    0x0000FF, 0x00FF00, 0xFF0000, 0);
//    if (icon) {
//	SDL_WM_SetIcon(icon, cciconmask);
//	SDL_FreeSurface(icon);
//    } else
//	warn("couldn't create icon surface: %s", SDL_GetError());
//
//    return _sdltimerinitialize(showhistogram)
//	&& _sdltextinitialize()
//	&& _sdltileinitialize()
//	&& _sdlinputinitialize()
//	&& _sdloutputinitialize(fullscreen)
//	&& _sdlsfxinitialize(silence, soundbufsize);
//}
int oshwinitialize(int silence, int soundbufsize,
		   int showhistogram, int fullscreen)
{
    SDL_Surface	       *icon;

    sdlg.eventupdatefunc = _eventupdate;

    if (SDL_Init(SDL_INIT_VIDEO) < 0) {
	errmsg(NULL, "Cannot initialize SDL system: %s\n", SDL_GetError());
	return FALSE;
    }
    //atexit(shutdown);

#ifdef PLATFORM_GP2X
    SDL_ShowCursor(SDL_DISABLE);
#endif

#ifdef PLATFORM_PC
	//dks debug
	printf("shouldn't happen on GP2X\n");
	
    setsubtitle(NULL);

    icon = SDL_CreateRGBSurfaceFrom(cciconimage, CXCCICON, CYCCICON,
				    32, 4 * CXCCICON,
				    0x0000FF, 0x00FF00, 0xFF0000, 0);
    if (icon) {
	SDL_WM_SetIcon(icon, cciconmask);
	SDL_FreeSurface(icon);
    } else
	warn("couldn't create icon surface: %s", SDL_GetError());
#endif //PLATFORM_PC




    return _sdltimerinitialize(showhistogram)
	&& _sdltextinitialize()
	&& _sdltileinitialize()
	&& _sdlinputinitialize()
	&& _sdloutputinitialize(fullscreen)
	&& _sdlsfxinitialize(silence, soundbufsize);
}

//DKS new delay function that allows us to wait but also update sound
void controlleddelay(int milsecs)
{
	int x;
	for (x = 0; x < milsecs; x += 10)
	{
		updatesound();
		SDL_Delay(10);
	}	
}


/* The real main().
 */
int main(int argc, char *argv[])
{
    //DKS - memory debugging
    //mwDoFlush(1);

    return tworld(argc, argv);
}
